
#ifndef KDEVPLATFORMUTIL_EXPORT_H
#define KDEVPLATFORMUTIL_EXPORT_H

#ifdef KDEVPLATFORMUTIL_STATIC_DEFINE
#  define KDEVPLATFORMUTIL_EXPORT
#  define KDEVPLATFORMUTIL_NO_EXPORT
#else
#  ifndef KDEVPLATFORMUTIL_EXPORT
#    ifdef KDevPlatformUtil_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMUTIL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMUTIL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMUTIL_NO_EXPORT
#    define KDEVPLATFORMUTIL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMUTIL_DEPRECATED
#  define KDEVPLATFORMUTIL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMUTIL_DEPRECATED_EXPORT
#  define KDEVPLATFORMUTIL_DEPRECATED_EXPORT KDEVPLATFORMUTIL_EXPORT KDEVPLATFORMUTIL_DEPRECATED
#endif

#ifndef KDEVPLATFORMUTIL_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMUTIL_DEPRECATED_NO_EXPORT KDEVPLATFORMUTIL_NO_EXPORT KDEVPLATFORMUTIL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMUTIL_NO_DEPRECATED
#    define KDEVPLATFORMUTIL_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMUTIL_EXPORT_H */
