
#ifndef KDEVPLATFORMDOCUMENTATION_EXPORT_H
#define KDEVPLATFORMDOCUMENTATION_EXPORT_H

#ifdef KDEVPLATFORMDOCUMENTATION_STATIC_DEFINE
#  define KDEVPLATFORMDOCUMENTATION_EXPORT
#  define KDEVPLATFORMDOCUMENTATION_NO_EXPORT
#else
#  ifndef KDEVPLATFORMDOCUMENTATION_EXPORT
#    ifdef KDevPlatformDocumentation_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMDOCUMENTATION_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMDOCUMENTATION_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMDOCUMENTATION_NO_EXPORT
#    define KDEVPLATFORMDOCUMENTATION_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMDOCUMENTATION_DEPRECATED
#  define KDEVPLATFORMDOCUMENTATION_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMDOCUMENTATION_DEPRECATED_EXPORT
#  define KDEVPLATFORMDOCUMENTATION_DEPRECATED_EXPORT KDEVPLATFORMDOCUMENTATION_EXPORT KDEVPLATFORMDOCUMENTATION_DEPRECATED
#endif

#ifndef KDEVPLATFORMDOCUMENTATION_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMDOCUMENTATION_DEPRECATED_NO_EXPORT KDEVPLATFORMDOCUMENTATION_NO_EXPORT KDEVPLATFORMDOCUMENTATION_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMDOCUMENTATION_NO_DEPRECATED
#    define KDEVPLATFORMDOCUMENTATION_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMDOCUMENTATION_EXPORT_H */
