
#ifndef KDEVPLATFORMSERIALIZATION_EXPORT_H
#define KDEVPLATFORMSERIALIZATION_EXPORT_H

#ifdef KDEVPLATFORMSERIALIZATION_STATIC_DEFINE
#  define KDEVPLATFORMSERIALIZATION_EXPORT
#  define KDEVPLATFORMSERIALIZATION_NO_EXPORT
#else
#  ifndef KDEVPLATFORMSERIALIZATION_EXPORT
#    ifdef KDevPlatformSerialization_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMSERIALIZATION_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMSERIALIZATION_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMSERIALIZATION_NO_EXPORT
#    define KDEVPLATFORMSERIALIZATION_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMSERIALIZATION_DEPRECATED
#  define KDEVPLATFORMSERIALIZATION_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMSERIALIZATION_DEPRECATED_EXPORT
#  define KDEVPLATFORMSERIALIZATION_DEPRECATED_EXPORT KDEVPLATFORMSERIALIZATION_EXPORT KDEVPLATFORMSERIALIZATION_DEPRECATED
#endif

#ifndef KDEVPLATFORMSERIALIZATION_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMSERIALIZATION_DEPRECATED_NO_EXPORT KDEVPLATFORMSERIALIZATION_NO_EXPORT KDEVPLATFORMSERIALIZATION_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMSERIALIZATION_NO_DEPRECATED
#    define KDEVPLATFORMSERIALIZATION_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMSERIALIZATION_EXPORT_H */
