
#ifndef KDEVPLATFORMINTERFACES_EXPORT_H
#define KDEVPLATFORMINTERFACES_EXPORT_H

#ifdef KDEVPLATFORMINTERFACES_STATIC_DEFINE
#  define KDEVPLATFORMINTERFACES_EXPORT
#  define KDEVPLATFORMINTERFACES_NO_EXPORT
#else
#  ifndef KDEVPLATFORMINTERFACES_EXPORT
#    ifdef KDevPlatformInterfaces_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMINTERFACES_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMINTERFACES_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMINTERFACES_NO_EXPORT
#    define KDEVPLATFORMINTERFACES_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMINTERFACES_DEPRECATED
#  define KDEVPLATFORMINTERFACES_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMINTERFACES_DEPRECATED_EXPORT
#  define KDEVPLATFORMINTERFACES_DEPRECATED_EXPORT KDEVPLATFORMINTERFACES_EXPORT KDEVPLATFORMINTERFACES_DEPRECATED
#endif

#ifndef KDEVPLATFORMINTERFACES_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMINTERFACES_DEPRECATED_NO_EXPORT KDEVPLATFORMINTERFACES_NO_EXPORT KDEVPLATFORMINTERFACES_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMINTERFACES_NO_DEPRECATED
#    define KDEVPLATFORMINTERFACES_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMINTERFACES_EXPORT_H */
