// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDEVELOP_VERSION_H
#define KDEVELOP_VERSION_H

#define KDEVELOP_VERSION_STRING "6.6.260800"
#define KDEVELOP_VERSION_MAJOR 6
#define KDEVELOP_VERSION_MINOR 6
#define KDEVELOP_VERSION_PATCH 260800
#define KDEVELOP_VERSION ((6<<16)|(6<<8)|(260800))

#endif
